import gleam/dynamic/decode.{type Decoder}
import gleam/time/timestamp.{type Timestamp}
import spacetraders_models/internal/time
import spacetraders_models/ship_symbol.{type ShipSymbol}
import spacetraders_models/trade_symbol.{type TradeSymbol}
import spacetraders_models/waypoint_symbol.{type WaypointSymbol}

pub type ShipModificationTransaction {
  ShipModificationTransaction(
    waypoint_symbol: WaypointSymbol,
    ship_symbol: ShipSymbol,
    trade_symbol: TradeSymbol,
    total_price: Int,
    timestamp: Timestamp,
  )
}

pub fn decoder() -> Decoder(ShipModificationTransaction) {
  use waypoint_symbol <- decode.field(
    "waypointSymbol",
    waypoint_symbol.decoder(),
  )
  use ship_symbol <- decode.field("shipSymbol", ship_symbol.decoder())
  use trade_symbol <- decode.field("tradeSymbol", trade_symbol.decoder())
  use total_price <- decode.field("totalPrice", decode.int)
  use timestamp <- decode.field("timestamp", time.rfc3339_timestamp_decoder())
  decode.success(ShipModificationTransaction(
    waypoint_symbol:,
    ship_symbol:,
    trade_symbol:,
    total_price:,
    timestamp:,
  ))
}
