import gleam/dynamic/decode.{type Decoder}
import gleam/json.{type Json}
import gleam/time/calendar
import gleam/time/timestamp.{type Timestamp}
import spacetraders_models/internal/time
import spacetraders_models/survey_deposit.{type SurveyDeposit}
import spacetraders_models/survey_signature.{type SurveySignature}
import spacetraders_models/survey_size.{type SurveySize}
import spacetraders_models/waypoint_symbol.{type WaypointSymbol}

pub type Survey {
  Survey(
    signature: SurveySignature,
    symbol: WaypointSymbol,
    deposits: List(SurveyDeposit),
    expiration: Timestamp,
    size: SurveySize,
  )
}

pub fn decoder() -> Decoder(Survey) {
  use signature <- decode.field("signature", survey_signature.decoder())
  use symbol <- decode.field("symbol", waypoint_symbol.decoder())
  use deposits <- decode.field(
    "deposits",
    decode.list(survey_deposit.decoder()),
  )
  use expiration <- decode.field("expiration", time.rfc3339_timestamp_decoder())
  use size <- decode.field("size", survey_size.decoder())
  decode.success(Survey(signature:, symbol:, deposits:, expiration:, size:))
}

pub fn encode(survey: Survey) -> Json {
  json.object([
    #("signature", survey_signature.encode(survey.signature)),
    #("symbol", waypoint_symbol.encode(survey.symbol)),
    #("deposits", json.array(survey.deposits, survey_deposit.encode)),
    #(
      "expiration",
      json.string(timestamp.to_rfc3339(survey.expiration, calendar.utc_offset)),
    ),
    #("size", survey_size.encode(survey.size)),
  ])
}
